@@ -3,7 +3,7 @@
 #ifndef cmCommand_h
 #define cmCommand_h
 
-#include "cmConfigure.h"
+#include "cmConfigure.h" // IWYU pragma: keep
 
 #include <string>
 #include <vector>
@@ -24,21 +24,19 @@ struct cmListFileArgument;
  */
 class cmCommand
 {
-  CM_DISABLE_COPY(cmCommand)
-
 public:
   /**
    * Construct the command. By default it has no makefile.
    */
-  cmCommand()
-    : Makefile(CM_NULLPTR)
-  {
-  }
+  cmCommand() = default;
 
   /**
    * Need virtual destructor to destroy real command type.
    */
-  virtual ~cmCommand() {}
+  virtual ~cmCommand() = default;
+
+  cmCommand(cmCommand const&) = delete;
+  cmCommand& operator=(cmCommand const&) = delete;
 
   /**
    * Specify the makefile.
@@ -90,7 +88,7 @@ public:
   void SetError(const std::string& e);
 
 protected:
-  cmMakefile* Makefile;
+  cmMakefile* Makefile = nullptr;
 
 private:
   std::string Error;
